/*
 *  Author: Enrico Forestieri <forenr@tlc.unipr.it>
 *  Created:  Sat Sep  4 16:47:24 2004
 *  Modified: Fri Jun 10 09:51:41 2005
 *     - If no flags given, -q implied.
 *  Modified: Sun Apr 16 20:30:36 2006
 *     - Account for spaces in gsprint path.
 *
 *  Modify and redistribute freely. Use at your own risk.
 *  This is distributed as part of the LyX/Cygwin package.
 *
 *  Program to print postscript files to a windows printer through the cygwin
 *  lpr or gsprint ( http://www.cs.wisc.edu/~ghost/gsview/gsprint.htm ).
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/cygwin.h>

#define LINSIZE 260
#define BUFSIZE 400
#define MAXNVER 10

int main(int ac, char **av)
{
    int c;
    extern char *optarg;
    extern int optind;
    int Dflg=0, Pflg=0, hflg=0, lflg=0, qflg=0;
    int errflg=0;
    FILE *fp;
    char *device=NULL;
    char *file=NULL;
    char pbuf[LINSIZE];
    char cbuf[BUFSIZE];
    char gsprint[LINSIZE];

    while ((c = getopt(ac, av, "DP:d:hlq")) != EOF)
    {
	switch (c)
	{
	    case 'D':
		Dflg++;
		break;
	    case 'P':
	    case 'd':
		Pflg++; 
		device = optarg;
		break;
	    case 'h':
		hflg++;
		break;
	    case 'l':
		lflg++;
		break;
	    case 'q':
		qflg++;
		break;
	    default:
		errflg++;
	}
    }

    if (errflg || (qflg && (Dflg || Pflg || hflg || lflg)))
    {
	fprintf(stderr,"Prints the given files to a windows printer by using the cygwin lpr or gsprint.\n\n");
	fprintf(stderr,"Usage: lp [-D] [-d device ] [-h] [-l] [-P device ] <files>...\n");
	fprintf(stderr,"   or: lp -q <postscript files>...\n\n");
	fprintf(stderr,"With the first form, merely calls lpr for each given file (see manpage for lpr).\n");
	fprintf(stderr,"With the second form (or if no switches are given), calls gsprint showing the\nprinter setup dialog. If gsprint is not found, defaults to lpr with no args.\n");
	exit(1);
    }

    if (!Dflg && !Pflg && !hflg && !lflg)
	qflg++;

    if (qflg)
    {	/* Look for gsview in the registry */
	gsprint[0] = 0;
	if ((fp=popen("regtool -q list /HKLM/SOFTWARE/Ghostgum/Gsview","r")))
	{   /* Scan all versions, keep the last good one */
	    int v=0;
	    char *ver[MAXNVER];
	    while (fgets(pbuf,sizeof(pbuf),fp) && v<MAXNVER)
	    {
		pbuf[strlen(pbuf)-1] = 0;
		if (pbuf[0])
		{
		    ver[v] = malloc(strlen(pbuf)+1);
		    strcpy(ver[v],pbuf);
		    v++;
		}
	    }
	    pclose(fp);
	    if (v)
	    {
		int i,l;
		strcpy(cbuf,"regtool -q get \"/HKLM/SOFTWARE/Ghostgum/Gsview/");
		l=strlen(cbuf);
		for (i=0; i<v; i++)
		{
		    cbuf[l] = 0;
		    strcat(cbuf,ver[i]);
		    strcat(cbuf,"\"");
		    if ((fp=popen(cbuf,"r")))
		    {
			if (fgets(pbuf,sizeof(pbuf),fp))
			{
			    char tbuf[LINSIZE];
			    pbuf[strlen(pbuf)-1] = 0;
			    if (pbuf[0])
			    {
				struct stat st;
				cygwin_conv_to_posix_path(pbuf,tbuf);
				strcat(tbuf,"/gsview/gsprint.exe");
				if (stat(tbuf,&st)==0)
				    strcpy(gsprint,tbuf);
			    }
			}
			pclose(fp);
		    }
		    free(ver[i]);
		}
	    }
	}
	if (gsprint[0])
	{
	    strcpy(cbuf,"\"");
	    strcat(cbuf,gsprint);
	    strcat(cbuf,"\" -q -query ");
	}
	else
	{   /* No gsview, defaults to lpr */
	    strcpy(cbuf,"/usr/bin/lpr ");
	}
    }
    else
    {
	strcpy(cbuf,"/usr/bin/lpr");
	if (Dflg)
	    strcat(cbuf," -D");
	if (Pflg)
	{
	    strcat(cbuf," -P ");
	    strcat(cbuf,device);
	}
	if (hflg)
	    strcat(cbuf," -h");
	if (lflg)
	    strcat(cbuf," -l");
	strcat(cbuf," ");
    }
    
    if (optind==ac)
    {	/* Read from stdin and create a temp file */
	file="/tmp/gsprint.ps";
	if ((fp=fopen(file,"w")))
	{
	    while (fgets(pbuf,sizeof(pbuf),stdin))
		fputs(pbuf,fp);
	    fclose(fp);
	}
	else
	{
	    fprintf(stderr, "%s: cannot open temporary file\n", av[0]);
	    exit(1);
	}
	cygwin_conv_to_win32_path(file, pbuf);
	strcat(cbuf,"'");
	strcat(cbuf,pbuf);
	strcat(cbuf,"'");
	system(cbuf);
	remove(file);
    }
    else
    {	/* Remaining args on cmd line are treated as files to print */
	int l = strlen(cbuf);
	while (optind<ac)
	{
	    cbuf[l] = 0;
	    cygwin_conv_to_win32_path(av[optind], pbuf);
	    strcat(cbuf,"'");
	    strcat(cbuf,pbuf);
	    strcat(cbuf,"'");
	    system(cbuf);
	    optind++;
	}
    }

    return 0;
}
